//
//  AMSDKMeetingManager.h
//  AliMeetingSDK
//
//  Created by xuanzhen.zjw on 2019/8/15.
//  Copyright © 2019 Alibaba. All rights reserved.
//

#import <Foundation/Foundation.h>

/**
 * 会议结束原因枚举
 */
typedef NS_ENUM(NSInteger, AMSDKFinishCode) {
    //! 用户主动退会
    AMSDKFinishCode_NORMAL_QUIT                 =   0,
    //! 信令错误
    AMSDKFinishCode_SIGNALING_ERROR             =   1,
    //! 会议已取消
    AMSDKFinishCode_MEETING_CANCELLED           =   2,
    //! 用户被移除会议
    AMSDKFinishCode_REMOVED_FROM_MEETING        =   3,
    //! 用户通过其它端入会，当前端被强制退会
    AMSDKFinishCode_HANGUP_BY_SELF              =   4,
    //! 用户被其他人挂断
    AMSDKFinishCode_HANGUP_BY_SERVER            =   5,
    //! 信令连接失败
    AMSDKFinishCode_SIGNALING_CONNECT_FAILED    =   6,
    //! ICE连接失败
    AMSDKFinishCode_ICE_CONNECT_FAILED          =   7
};

/**
 * 错误码
 */
typedef NS_ENUM(NSInteger, AMSDKErrorCode) {
    //! 操作成功
    AMSDKErrorCode_SUCCEED                              =   0,
    //! 错误的调用，接口调用时机不对，比如已经入会的情况下继续调用入会接口
    AMSDKErrorCode_INVALID_INVOCATION                   =   1,
    //！非法参数，接口调用参数不正确
    AMSDKErrorCode_ILLEGAL_PARAMETERS                   =   2,
    //！会议状态异常，比如成员列表为空，自己不在会议列表等情况会导致入会失败
    AMSDKErrorCode_UNEXPECTED_STATUS                    =   3,
    //! 没有录音、相机权限
    AMSDKErrorCode_PERMISSION_NOT_GRANTED               =   4,
    //! 会议初始化失败，如无法获取会议信息，拉取配置
    AMSDKErrorCode_INITIALIZE_FAILED                    =   5,
    //! 入会超时
    AMSDKErrorCode_JOIN_MEETING_TIMEOUT                 =   6,
    //! 无法连接媒体服务器
    AMSDKErrorCode_MEDIA_SERVER_CONNECT_FAILED          =   7,
    //! 媒体协商失败
    AMSDKErrorCode_MEDIA_NEGOTIATION_FAILED             =   8,
    //! 参会人数超过限制
    AMSDKErrorCode_MEDIA_ROOM_FULL                      =   9,
    //! ROOM被锁定
    AMSDKErrorCode_MEDIA_ROOM_LOCKED                    =   10,
    //! ROOM不存在
    AMSDKErrorCode_MEDIA_ROOM_NOT_EXIST                 =   11
};

/**
 * 列表改变事件
 */
typedef NS_ENUM(NSInteger, AMSDKClientListEvent) {
    //! 初始化返回列表
    AMSDKClientListEvent_INIT       =   0,
    //! 会中添加人员列表
    AMSDKClientListEvent_ADD        =   1,
    //! 会中移除人员列表
    AMSDKClientListEvent_REMOVE     =   2
};

/**
 * Meeting Client状态事件
 */
typedef NS_ENUM(NSInteger, AMSDKStatusEvent) {
    //! 在线/离线
    AMSDKStatusEvent_ONLINE                 =   0,
    //! 打开/关闭视频
    AMSDKStatusEvent_VIDEO                  =   1,
    //! 打开/关闭音频
    AMSDKStatusEvent_AUDIO                  =   2,
    //! 开始/结束发言
    AMSDKStatusEvent_TALKING                =   3,
    //! 开始/结束响铃
    AMSDKStatusEvent_RINGING                =   4,
    //! 网络质量变化
    AMSDKStatusEvent_NETWORK_QUALITY        =   5,
    //! 切换主讲人
    AMSDKStatusEvent_MAIN_SPEAKER           =   6,
    //! 流状态发生改变
    AMSDKStatusEvent_STREAM_READY           =   7
};

/**
 * 媒体类型
 */
typedef NS_ENUM(NSInteger, AMSDKMediaType) {
    //! 音频
    AMSDKMediaType_AUDIO      =   0,
    //! 视频
    AMSDKMediaType_VIDEO      =   1,
    //! 音视频
    AMSDKMediaType_ALL        =   2
};

//! 会议状态
typedef NS_ENUM(NSInteger, AMSDKMeetingStatus) {
    //! 初始状态
    AMSDKMeetingStatus_IDLE          =  0,
    //! 连接中
    AMSDKMeetingStatus_JOINING       =  1,
    //! 已连接
    AMSDKMeetingStatus_JOINED        =  2,
    //! 重连中
    AMSDKMeetingStatus_REJOINING     =  3,
    //! 连接出错
    AMSDKMeetingStatus_ERROR         =  4,
    //! 会议终止
    AMSDKMeetingStatus_FINISHED      =  5
};

NS_ASSUME_NONNULL_BEGIN

@class AMSDKMeetingClient;
@class AMSDKMediaDeviceManager;

/**
 * AMSDKMeetingCallback用于会议回调，所有回调方法均在主线程执行
 */
@protocol AMSDKMeetingCallback <NSObject>

@optional

//! 会议状态
- (void)onMeetingStatusChange:(AMSDKMeetingStatus)meetingStatus;

//! 入会成功
- (void)onMeetingJoined;

//! 入会出现错误
- (void)onError:(AMSDKErrorCode)code reason:(NSString *)reason;

//! 参会列表发生变化
- (void)onClientListChange:(AMSDKClientListEvent)event list:(NSArray<AMSDKMeetingClient *> *)list;

//! 参会端发生变化
- (void)onClientStatusChange:(AMSDKMeetingClient *)client event:(AMSDKStatusEvent)event;

//! 媒体流状态改变，client 为自己，一般为带宽较低，服务端建议主动断流，需要用户手动断流，client 为其他人时，一般为服务端不给端发送对应client的流了，收到该回调时状态已经发生改变
- (void)onStreamStatusChange:(AMSDKMeetingClient *)client streamActive:(BOOL)streamActive;

//! 会议结束
- (void)onMeetingFinish:(AMSDKFinishCode)code reason:(NSString *)reason;

//! 收到DataChannel消息
- (void)onChannelMessage:(NSString *)msg;

 //! 啸叫状态改变
- (void)onWhistlingNoiseStatusChange;

@end

/**
 * 会议管理类
 * AMSDKMeetingManager描述整个会议控制，包括订阅和推送管理、加入和离开、共享桌面和结束共享、获取成员列表等。
 */
@interface AMSDKMeetingManager : NSObject

//! 会议状态
@property (nonatomic, assign, readonly) AMSDKMeetingStatus status;

//! 媒体设备管理对象，用于切换摄像头，开关扬声器等
@property (nonatomic, strong, readonly) AMSDKMediaDeviceManager *deviceManager;

//! 是否为MCU会议
@property (nonatomic, assign, readonly) BOOL isMCU;

//! 初始化方法
- (instancetype)initWithMeetingConfig:(NSDictionary *)config callback:(id<AMSDKMeetingCallback>)callback;

//! 通过此方法判断当前机器是否支持视频会议
+ (BOOL)supportsVideo;

//! 获取SDK版本号
+ (NSString *)getSDKVersion;

//! 加入会议
- (void)joinMeeting;

//! 自己退出会议
- (void)leaveMeeting;

//! 退出会议，并踢出已入会的所有人，会议发起者才有权限
- (void)finishMeeting;

//! 是否接受大流，其他自动切换为小流
- (void)enableHDVideo:(AMSDKMeetingClient *)client enabled:(BOOL)enabled;

//! 根据uuid查找client
- (AMSDKMeetingClient *)getClient:(NSString *)uuid;

//! 获取所有参会端
- (NSArray<AMSDKMeetingClient *> *)getAllClients;

//! 获取所有在线端
- (NSArray<AMSDKMeetingClient *> *)getOnlineClients;

//! 获取自身
- (AMSDKMeetingClient *)publisherClient;

//! 获取SDK版本号
- (NSString *)getSDKVersion;

//! 获取媒体统计信息
- (void)getMediaStats:(void (^)(NSDictionary * _Nullable stats))completionHandler;

//! Not supported
- (void)startShareScreen NS_UNAVAILABLE;

//! Not supported
- (void)stopShareScreen NS_UNAVAILABLE;

#pragma mark - 注意：下列初始化方法不可用

+ (instancetype)new NS_UNAVAILABLE;

- (instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
